# tofsimsapp.tcl is a plugin for VMD
#
# This is version 1.2 as of Tue Jun 21 14:07:21 PDT 2011
#
# tofsimsapp (or tofsimsplugin) is free software distributed under the
# GNU General Public License: http://www.gnu.org/licenses/gpl.html
# This TCL plugin for VMD is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
# Authors:
# Julia Doh and Gianluca Interlandi
#
# VMD can be downloaded from: http://www.ks.uiuc.edu/Research/vmd/
#
# USAGE:
# Start VMD and load the PDB of interest
# Open a Tk console from "Extensions" -> "Tk console"
# Make sure you cd into the directory where tofsimsapp.tcl is located
# Type: source tofsimsapp.tcl

# Print a banner giving credit where credit is due
puts "************************************************************"
puts "* Running ToF-SIMS app                                     *"
puts "* AUTHORS:                                                 *"
puts "* Julia Doh and Gianluca Interlandi                        *"
puts "* gianluca@uw.edu juliadoh@uw.edu                          *"
puts "************************************************************"

# Explain about direction of beam and placement of surface
puts ""
puts "In the default settings, the surface is parallel to the xz-plane,"
puts "which is perpendicular to the screen and below the molecule."
puts "The source of the ion beam is in the +y direction."
puts "This setting, as well as the depth of the hit layer,"
puts "can be changed in the top part of the plugin window."
puts "For help with VMD: http://www.ks.uiuc.edu/Research/vmd/current/ug/"
puts " "

#creates the window for the TK widgets that run the program
package require Tk
set w [toplevel ".textview"] 
wm title $w "ToF-SIMS"

set cback 0
#enable or disable R shortcut
button $w.shortcuty -text "Turn on R shortcut" -command {enabletrace; set cback 1}
button $w.shortcutn -text "Turn off R shortcut" -command {disabletrace; set cback 0}

#Radiobuttons to select the dimension targeted and min/max for ToF-SIMS. Default value is +y
label $w.di -text "Targeted side"
radiobutton $w.x -text +X -variable side -value +x
radiobutton $w.y -text +Y -variable side -value +y
radiobutton $w.z -text +Z -variable side -value +z
radiobutton $w.nx -text -X -variable side -value -x
radiobutton $w.ny -text -Y -variable side -value -y
radiobutton $w.nz -text -Z -variable side -value -z
radiobutton $w.count -text "# of blown residues" -variable onratio -value 0
radiobutton $w.ratio -text "blown residues/total residues per AA" -variable onratio -value 1
label $w.el -text "Protein Selection"
entry $w.e -textvariable selection -width 75
$w.y select
$w.count select

#Spinbox to change the depth. User can spin through values or enter their own. Default value is 10. 
label $w.ang -text "Depth in angstroms" 
set deeps [list 20 25 30 35 0 ]
spinbox $w.depth -from 0 -to 30 -textvariable spinlayer -values $deeps -wrap 1

#Button to run the program
button $w.calc -text "Graph" -command draw_histogram -width 10

#puts the widgets in a window so the user can interact with them
grid $w.di -row 0 -column 0 -columnspan 1 -rowspan 2
grid $w.x -row 0 -column 1 -columnspan 1 
grid $w.y -row 0 -column 2 -columnspan 1 
grid $w.z -row 0 -column 3 -columnspan 1 
grid $w.ang -row 0 -column 4 -columnspan 1
grid $w.shortcuty -row 0 -column 5 -columnspan 1
grid $w.nx -row 1 -column 1 -columnspan 1 
grid $w.ny -row 1 -column 2 -columnspan 1 
grid $w.nz -row 1 -column 3 -columnspan 1 
grid $w.depth -row 1 -column 4 -columnspan 1
grid $w.shortcutn -row 1 -column 5 -columnspan 1
grid $w.el -row 2 -column 0 -columnspan 1 
grid $w.e -row 2 -column 1 -columnspan 5   
grid $w.calc -row 3 -column 0 -columnspan 1
grid $w.count -row 3 -column 2 -columnspan 2
grid $w.ratio -row 3 -column 4 -columnspan 2

# Shortcut: runs the program if the R button is pushed
proc callback {varname args} { 
    upvar #0 $varname var
    if {$var =="rotate"} {
        draw_histogram
    }
}

#turns on callbacks to enable the R shortcut
proc enabletrace {} {
    global cback
    if {$cback == 0} {
        eval "trace add variable ::vmd_mouse_mode write callback"
    }
}

#disables the R shortcut
proc disabletrace {} {
    global cback
    eval "trace remove variable ::vmd_mouse_mode write callback"
    set cback 0
}

# Calls the routines to draw the histogram
proc draw_histogram {} {
    set histinput [tofsims]
    histogram $histinput 600 450
}

# Creates a histogram from the blown residue values
set ready 0
proc histogram {slist wid ht} {
    global ready
    global w
    global onratio
    array set scores $slist
    set resletter {A R N D C Q E G H I L K M F P S T W Y V}
    set nms [lsort -integer [array names scores]]
    catch {destroy .h} {}
    catch {destroy .b} {}
    canvas $w.h -width 614 -height $ht -bg white
    set nbars [array size scores] ;#how many histogram bars
    set nax [expr $nbars/2] ;# axis spacing
    set hwid [expr $wid/ $nbars]
    set i 0
    set m 0
    set yaxis {}
    set hmax -9999999 ;# largest y value
    set hmin  9999999 ;# smallest y value
    
    #finds maximum value to use for axis
   	if {$onratio == 0} {
        set q 3
        set axismax [lindex $slist 1]
        while {$q < 100} {
            if {[lindex $slist $q] > $axismax} {
                set axismax [lindex $slist $q]
            }
            incr q 2
        }
    } else {
		set axismax 1.00
	}
	
    while {$i<$nbars} {
        set f $scores([lindex $nms $i])
        set hmax [ expr {$f>$hmax} ? $f : $hmax]
        set hmin [ expr {$f<$hmin} ? $f : $hmin]
        incr i
    }
    
    if {$onratio == 0} {    
        set splitmax [split $axismax {}]
        set maxinteger [llength $splitmax]
    } else {
		set maxinteger 1
	}
    if {$maxinteger > 1} {
        set axisdiv [expr (int([expr $axismax/10])+1)]
        set axistop [expr $axisdiv*10]
        set u 0
        while {$u <= 30} {
            set axisentry [expr $axistop/5*$u]
            lappend yaxis $axisentry
            incr u
        }
        set axistop_pix [expr $ht-20-0.85*$ht*$axistop/$hmax]
        set tickmark_size [expr (430-$axistop_pix)/5]
    } else {
        set axistop $axismax
        set axisdiv $axismax
        set u 0
        if {$onratio == 0} {
            while {$u <= $axisdiv} {
                set axisentry $u
                lappend yaxis $axisentry
                incr u
            }
        } else {
			set yaxis {0 .2 .4 .6 .8 1}
	    }
        set axistop_pix [expr $ht-20-0.85*$ht*$axistop/$hmax]
        set tickmark_size [expr (430-$axistop_pix)/$axisdiv]
        if {$onratio == 1} {
			set tickmark_size [expr (430-$axistop_pix)/5]
		}
    }
    
    #y axis
    if {$onratio == 1} {
		set axistop 5
	}
    while {$m <= $axistop} {
        set yht [expr 430-($tickmark_size*$m)]
        $w.h create line 20 430 600 430 -fill black
        $w.h create line 20 0 20 430 -fill grey
        $w.h create text 10 $yht -text [lindex $yaxis $m]
        $w.h create line 20 $yht 600 $yht -fill #EEEEEE
        incr m    
    }
    if {$hmax>$hmin} {
        set i 0
        set nay 100
        while {$nay<$hmax} {
            set yp [expr $ht-0.75*$nay-20]
            incr nay 100
        }
        set nax 5
 
    #histogram code
    while {$i<$nbars} {
        set x [expr $hwid*$i]
        set rhs [expr $x+$hwid]
        set f [expr $ht-20-.85*$ht*$scores([lindex $nms $i])/$hmax]
        if {$onratio == 0} {
			set color #3399FF
		} else {
			set color #66CC66
		}
        $w.h create rectangle  [expr $x+20] [expr $ht-20] $rhs $f -fill $color
        $w.h create text [expr $x+25] [expr $ht-10] -text [lindex $resletter $i]
        $w.h create text [expr $x+25] [expr $ht-440] -text [lindex $resletter $i]
        $w.h create text [expr $x+25] [expr $f-10] -text $scores([lindex $nms $i])
        incr i
        incr x $hwid
    }
    
    update
    grid $w.h -row 4 -column 0 -columnspan 6
    }
 }

proc find_minmax {reslist numres sign} {
	set minmax [lindex $reslist 0]
    set minmax_el 0
    for {set i 1} {$i<$numres} {incr i} {
        if {$sign =="+"} {
        if {[lindex $reslist $i] > $minmax} {
                set minmax [lindex $reslist $i]
                set minmax_el $i
            }
        } else {
            if {[lindex $reslist $i] < $minmax} {
                set minmax [lindex $reslist $i]
                set minmax_el $i
            }
        }
   }
   set minmaxellist "$minmax $minmax_el"
   return $minmaxellist
}

proc find_numres_reslist {R variable protein} {
	set numres [llength [[atomselect top "$protein"] get resid]]
	for {set i 0} {$i<$numres} {incr i} {
        set geom_center_res [coordtrans $R [measure center [atomselect top "residue $i"]]]
        lappend v_list [lindex $geom_center_res $variable] 
    }
    set numreslist "$numres $v_list"
    return $numreslist
}

# Main function: compute the residues blown off in a TOPS experiment
proc tofsims {} {
    global spinlayer
    global side
    global w
    global histinput
    global selection
    global onratio
    global numres
    
    set geom_center_res {}
    set v_list {}
    set layer $spinlayer
    set histinput {}
    set side_list [split $side ""]
    set sign [lindex $side_list 0]
    set dir [lindex $side_list 1]
    set lsel [llength $selection]
    if {$dir == "x"} {
        set variable 0
    }
    if {$dir == "y"} {
        set variable 1
    }
    if {$dir == "z"} {
        set variable 2
    }
    #destroys the old histogram and the old list so a new graph and axis can load
    destroy $w.h
    
    # Find the transformation matrix into world coordinates
    set R [lindex [molinfo top get rotate_matrix] 0]
    
    #calculates true number of residues
    set protein "name CA"
    set truenumreslist [find_numres_reslist $R $variable $protein]   
    set truenumres [lindex $truenumreslist 0]
    set truereslist [lrange $truenumreslist 1 end]

    
    # Changes the selection based on user input and gets the corresponding number of residues
    if {$lsel > 0} {
		set protein "($selection) and name CA"
		set usernumreslist [find_numres_reslist $R $variable $protein]   
        set usernumres [lindex $usernumreslist 0]
        set userreslist [lrange $usernumreslist 1 end]
	}
	
	# Find the true element with the maximum/minimum x/y/z coordinate
    set trueminmaxellist [find_minmax $truereslist $truenumres $sign]
    set trueminmax [lindex $trueminmaxellist 0]
    set trueminmax_el [lindex $trueminmaxellist 1]
   
    # Find the user element with the maximum/minimum x/y/z coordinate
    if {$lsel > 0} {
        set userminmaxellist [find_minmax $userreslist $usernumres $sign]
        set userminmax [lindex $userminmaxellist 0]
        set userminmax_el [lindex $userminmaxellist 1]
    }

     # calculates delta variable
    if {$lsel > 0} {
        if {$sign == "+"} {
            set delta [expr abs($trueminmax-$userminmax)] 
	    } else {
	        set delta [expr abs($userminmax-$trueminmax)]
	    }
	    set layer [expr $spinlayer+$delta]
	}
	
	#Reports the residue with the minimum/maximum x/y/z value
    set res [lindex [[atomselect top "name CA and residue $trueminmax_el"] get {resname resid}] 0]
    if {$sign == "+"} {
        set topbottom "top"
    } else {
        set topbottom "bottom"
    }
    puts "Residue at the $topbottom $res \n"
    
    # Find all residues whose sidechain is within layer of trueminmax_el
    set blown_res {}
    for {set i 0} {$i<$truenumres} {incr i} {
    # calculate the distance between the residue and a plane through max x/y/z
        if {$sign == "+"} {
            set dist [expr $trueminmax - [lindex $truereslist $i]]
            if {$dist <= $layer} {
                set res [lindex [[atomselect top "name CA and residue $i"] get {resname resid}] 0]
                lappend blown_res $res
            }
        } else {
            set dist [expr [lindex $truereslist $i] - $trueminmax]
                if {$dist <= $layer} {
                    set res [lindex [[atomselect top "name CA and residue $i"] get {resname resid}] 0]
                    lappend blown_res $res
                }
        }
    }
    
    puts "$blown_res \n"
       
    # Find out the number of blown residues
    set num_blown [llength $blown_res]
    puts "Number of blown residues $num_blown \n"
    for {set i 0} {$i < $num_blown} {incr i} {
        lappend resid_list [lindex [lindex $blown_res $i] 1]
    }
    
    #Calculates the total number of residues
    set numres [llength [[atomselect top "name CA"] get resid]]
    for {set i 0} {$i < $numres} {incr i} {
        set totalres [lindex [[atomselect top "name CA"] get {resname}] $i]
        lappend totalreslist $totalres
	}

    #creates an array from which a histogram can be made
    set j 0
    foreach resname {ALA ARG ASN ASP CYS GLN GLU GLY HIS ILE LEU LYS MET PHE PRO SER THR TRP TYR VAL} {
        set number [llength [lsearch -glob -all $blown_res $resname*]]
        if {$onratio == 0 } {
			set num $number
			puts "$resname: $num"
		} else {
	        set blownnum $number
            set totalresi [llength [lsearch -glob -all $totalreslist $resname*]]
			if {$totalresi == 0} {
			     set totalresi 1
			}	
			set num [expr 1.0*round([expr 100*$number/$totalresi])/100]
	        puts "$resname: $blownnum / $totalresi = $num"
	    }
    lappend histinput $j $num
    set j [expr $j+1]
    }

    # Add a representation to display the blown residues
    if {[lindex [lindex [split [mol list top] "\n"] 2] 2] == 1} {
        mol addrep top
        mol modstyle 1 top Licorice
        mol modcolor 1 top Name
    }

    mol modselect 1 top "protein and resid $resid_list and not hydrogen"
    #histogram $histinput 600 450
    return $histinput
    selection {}
}
